@@ -1,6 +1,6 @@
 #
-# Author:: Christopher Brown (<cb@opscode.com>)
-# Copyright:: Copyright (c) 2009 Opscode, Inc.
+# Author:: Christopher Brown (<cb@chef.io>)
+# Copyright:: Copyright (c) 2009-2018 Chef Software, Inc.
 # License:: Apache License, Version 2.0
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -16,7 +16,7 @@
 # limitations under the License.
 #
 
-require "mixlib/authentication"
+require_relative "../authentication"
 require "openssl"
 
 module Mixlib
@@ -27,9 +27,7 @@ module Mixlib
         def hash_file(f, digest = OpenSSL::Digest::SHA1)
           digester = digest.new
           buf = ""
-          while f.read(16384, buf)
-            digester.update buf
-          end
+          digester.update buf while f.read(16384, buf)
           ::Base64.encode64(digester.digest).chomp
         end
 
